//ver 1.5
//last change 20.01.2025
unit hiMediaPlayerEx;

interface

uses Media,Windows,Kol,Share,Debug,Messages,mmsystem;

type
  ThiMediaPlayerEx = class(TDebug)
   private
    FMedia: PKOLMediaPlayer;
    WndH: HWND;
    Th: PTimer;
    procedure OnTimer(Sender:PObj);
    procedure SetSpeed(Value:integer);
    procedure SetVolume(Value:integer);
    procedure SetChannels(Value:byte);
    procedure SetVideoScale(Value:byte);
    procedure PlayMedia(pos:integer);
    procedure StopMedia;
   public
    _prop_Filename:string;
    _prop_Alias:string;
    _prop_StartPos:integer;
    _prop_FinishPos:integer;
    _prop_Interval:integer;
    _prop_AutoPlay:boolean;
    _prop_Replay:boolean;
    _prop_AllPlay:boolean;
    _prop_Channels:byte;

    _data_FileName:THI_Event;
    _data_Alias:THI_Event;
    _data_Handle:THI_Event;
    _data_DisplayRect:THI_Event;
    _data_Volume:THI_Event;
    _event_onOpen:THI_Event;
    _event_onPosition:THI_Event;
    _event_onEndPlay:THI_Event;
    _event_onError:THI_Event;

    constructor Create{(Control:PControl)};
    destructor Destroy; override;
    procedure _work_doPlay(var _Data:TData; Index:word);
    procedure _work_doStop(var _Data:TData; Index:word);
    procedure _work_doPause(var _Data:TData; Index:word);
    procedure _work_doPosition(var _Data:TData; Index:word);
    procedure _work_doOpen(var _Data:TData; Index:word);
    procedure _work_doClose(var _Data:TData; Index:word);
    procedure _work_doStartPos(var _Data:TData; Index:word);
    procedure _work_doFinishPos(var _Data:TData; Index:word);    
    procedure _work_doAutoPlay(var _Data:TData; Index:word);
    procedure _work_doReplay(var _Data:TData; Index:word);
    procedure _work_doAllPlay(var _Data:TData; Index:word);
    procedure _work_doSpeed(var _Data:TData; Index:word);
    procedure _work_doVolume(var _Data:TData; Index:word);
    procedure _work_doChannels(var _Data:TData; Index:word);
    procedure _work_doScale(var _Data:TData; Index:word);
    procedure _work_doSetDisplayRect(var _Data:TData; Index:word);

    procedure _var_Position(var _Data:TData; Index:word);
    procedure _var_Length(var _Data:TData; Index:word);
    procedure _var_Width(var _Data:TData; Index:word);
    procedure _var_Height(var _Data:TData; Index:word);
    procedure _var_State(var _Data:TData; Index:word);
    procedure _var_FrameRate(var _Data:TData; Index:word);   

    property _prop_Speed:integer write SetSpeed;
    property _prop_Volume:integer write SetVolume;
    property _prop_Scale:byte write SetVideoScale;
  end;

implementation                                 

constructor ThiMediaPlayerEx.Create;
begin
try
   inherited Create;
   FMedia := NewKOLMediaPlayer('',ReadHandle);
   FMedia.Display := ReadHandle;

   Th := NewTimer(1000);
   Th.Interval := _prop_Interval;
   Th.OnTimer := OnTimer;
except end;
end;

procedure ThiMediaPlayerEx.OnTimer;
  procedure NeedReplay;
  begin
   _hi_OnEvent(_event_onEndPlay);
   if _prop_Replay then
    begin
     FMedia.Position := max(0, _prop_StartPos);
     PlayMedia(-1);
    end;
  end;
begin
try
    _hi_OnEvent(_event_onPosition, FMedia.Position); 
    if FMedia.State = mpPlaying then exit; 
    th.Enabled := false;
    case _prop_FinishPos of
       -1: if FMedia.Position = FMedia.Length then NeedReplay;
       else if _prop_FinishPos = FMedia.Position then NeedReplay;
    end; 
except end;
end;

procedure ThiMediaPlayerEx.SetSpeed;
begin
try
    FMedia.DGV_Speed := Value;
except end;
end;

procedure ThiMediaPlayerEx.SetVolume;
begin
try
   FMedia.Volume := Value;
except end;
end;

procedure ThiMediaPlayerEx.SetChannels;
    procedure ChangeChannel(Chn:TSoundChannels; b: boolean);
    begin
       FMedia.AudioOn[Chn] := b;
    end;
begin
try
    case value of
     0: ChangeChannel([chLeft, chRight], False);
     1: begin 
         ChangeChannel([chLeft], True);
         ChangeChannel([chRight], False);
        end;
     2: begin
         ChangeChannel([chLeft], False);
         ChangeChannel([chRight], True);
        end;
     else ChangeChannel([chLeft, chRight], True); 
    end;  
except end;
end;

procedure ThiMediaPlayerEx.SetVideoScale;
begin
try
   FMedia.Scale := Value;
   if FMedia.State = mpPlaying then FMedia.Play(WndH, -1, _prop_FinishPos);// PlayMedia(-1);
except end;
end;

destructor ThiMediaPlayerEx.Destroy;
begin
try
   FMedia.Free;
   inherited Destroy;
except end;
end;


procedure ThiMediaPlayerEx.PlayMedia;
begin
try
    if FMedia.Play(WndH, pos, _prop_FinishPos) then
     begin
      Th.Enabled := true;
      _hi_OnEvent(_event_onPosition, FMedia.Position);
     end;
except end;
end;

procedure ThiMediaPlayerEx._work_doOpen;
var FName:string;
begin
try
   WndH := ReadInteger(_Data,_data_Handle,0);
   FName := ReadString(_Data,_data_FileName,_prop_FileName);
   if FileExists(FName) then
    begin
     FMedia.Alias := ReadString(_Data,_data_Alias,_prop_Alias);
     FMedia.FileName := FName;
	 if FMedia.Error <> 0 then _hi_OnEvent(_event_onError, FMedia.ErrorMessage);
	 FMedia.TimeFormat := tfMilliseconds;
     SetChannels(_prop_Channels);
     if _prop_AllPlay then
      begin
       _prop_StartPos := -1;
       _prop_FinishPos := -1;
      end;
     if _prop_AutoPlay then PlayMedia(_prop_StartPos)
      else
        begin
          FMedia.Play(WndH, _prop_StartPos, max(0, _prop_StartPos));
         _hi_OnEvent(_event_onPosition, FMedia.Position);
        end;
     _hi_OnEvent(_event_onOpen, FMedia.Length);
    end; 
except end;
end;

procedure ThiMediaPlayerEx._work_doPlay;
begin
try
    if FMedia.State = mpPlaying then exit; 
    PlayMedia(-1);
except end;
end;

procedure ThiMediaPlayerEx.StopMedia;
begin
try
   FMedia.Position := max(0, _prop_StartPos);
   _hi_OnEvent(_event_onPosition, FMedia.Position);
   FMedia.Stop;
except end;
end;

procedure ThiMediaPlayerEx._work_doStop;
begin
    StopMedia;
end;

procedure ThiMediaPlayerEx._work_doPause;
begin
try
   FMedia.Pause := not FMedia.Pause;
   th.Enabled := not FMedia.Pause;
   except end;
end;

procedure ThiMediaPlayerEx._work_doPosition;
begin
try
   if FMedia.Length > 0 then
    begin
     FMedia.Position := ToInteger(_Data);
     if Th.Enabled then PlayMedia(-1) //   
      else _hi_OnEvent(_event_onPosition, FMedia.Position);
    end;  
except end;
end;

procedure ThiMediaPlayerEx._work_doStartPos;
var PlayParm: TMCI_Play_Parms;
begin
    _prop_StartPos :=  ToInteger(_Data);
    PlayParm.dwFrom := _prop_StartPos;
    mciSendCommand(WndH, MCI_PLAY, MCI_FROM, Cardinal(@PlayParm));
    if FMedia.State = mpPlaying then 
     if FMedia.Position < _prop_StartPos then PlayMedia(_prop_StartPos)
      else PlayMedia(-1); 
end;

procedure ThiMediaPlayerEx._work_doFinishPos;
var PlayParm: TMCI_Play_Parms;
    pl: boolean;
begin
    if FMedia.State = mpPlaying then pl := true else pl := false;
    _prop_FinishPos :=  ToInteger(_Data);
    PlayParm.dwTo := _prop_FinishPos;
    mciSendCommand(WndH, MCI_PLAY, MCI_TO, Cardinal(@PlayParm));
    if pl then
     begin
      PlayMedia(-1);
      if (FMedia.Position >= _prop_FinishPos) then
       StopMedia;
     end
    else StopMedia;
end;

procedure ThiMediaPlayerEx._work_doAutoPlay;
begin
    _prop_AutoPlay :=  ReadBool(_Data);
end;

procedure ThiMediaPlayerEx._work_doReplay;
begin
    _prop_Replay :=  ReadBool(_Data);
end;

procedure ThiMediaPlayerEx._work_doAllPlay;
begin
    _prop_AllPlay :=  ReadBool(_Data);
end;

procedure ThiMediaPlayerEx._work_doSpeed;
var i: integer;
begin
    i := ToInteger(_Data);
    if (i < 0) then i := 0
     else if i > 2000 then i := 2000; 
    SetSpeed(i);
end;

procedure ThiMediaPlayerEx._work_doVolume;
var i: integer;
begin
    i := ToInteger(_Data);
    if (i < 0) then i := 0
     else if i > 1000 then i := 1000; 
    SetVolume(i);
end;


procedure ThiMediaPlayerEx._work_doChannels;
begin
    SetChannels(ToInteger(_Data)); 
end;

procedure ThiMediaPlayerEx._work_doScale;
var i: integer;
begin
    i := ToInteger(_Data);
    if (i < 0) then i := 0
     else if i > 2 then i := 2; 
    SetVideoScale(i);
end;


procedure ThiMediaPlayerEx._work_doClose;
begin       
   _work_doStop(_Data,0);
   FMedia.Close;
end;

procedure ThiMediaPlayerEx._work_doSetDisplayRect;
var dt:TData;
begin
  if Assigned(_data_DisplayRect.Event) then
   begin
    dt.Data_type := data_null;
    {$ifdef _DEBUG_}CallEvent(4,_data_DisplayRect,dt);{$endif}
    _data_DisplayRect.Event(dt,_data_DisplayRect.Index);
    {$ifdef _DEBUG_}CallEvent(3,_data_DisplayRect,dt);{$endif}
    if dt.Data_type = data_rect then
    FMedia.DisplayRect := (PRect(dt.idata)^);
   end;
end;

procedure ThiMediaPlayerEx._var_Position;
begin
    dtInteger(_Data,FMedia.Position);
end;

procedure ThiMediaPlayerEx._var_Length;
begin
    dtInteger(_Data,FMedia.Length);
end;

procedure ThiMediaPlayerEx._var_Width;
begin
    dtInteger(_Data,FMedia.Width);
end;

procedure ThiMediaPlayerEx._var_Height;
begin
    dtInteger(_Data,FMedia.Height);
end;

procedure ThiMediaPlayerEx._var_State;
begin
try
    case FMedia.State of
     mpStopped:   dtInteger(_Data,0);
     mpPlaying:   dtInteger(_Data,1);
     mpRecording: dtInteger(_Data,2);
     mpSeeking:   dtInteger(_Data,3);
     mpPaused:    dtInteger(_Data,4);
     mpOpen:      dtInteger(_Data,5);
     else         dtInteger(_Data,6);
    end; 
except end;
end;

procedure ThiMediaPlayerEx._var_FrameRate;
var StatusParm: TMCI_Status_Parms;
begin
    if FMedia.Width = 0 then
     begin
      dtReal(_Data, 0);
      exit;
     end; 
    StatusParm.dwItem := $400E and $7FFFFFFF; //MCI_STATUS_TIME_FORMAT = $400E
    if 0 = mciSendCommand(FMedia.DeviceID, MCI_STATUS,  MCI_WAIT or MCI_STATUS_ITEM, cardinal(@StatusParm))
    then dtReal(_Data, StatusParm.dwReturn / 1000)
    else dtReal(_Data, 0);
end;
end.
